(*************************************************************************
*                                                                        *
*    PACKAGE  MTXcalc  from  NUMERIK                                     *
*                                                                        *
**************************************************************************
    *                                                                    *
    *  J. Desteffani IA85                     01.01.1987 - 26.01.1987    *
    *                                                                    *
    *  p u r p o s e :                                                   *
    *    Dieses Package fuehrt die wichtigsten Matrizenrechenoperationen *
    *    aus. Einige Berechnungen werden im Package MATPACK.PAS durch -  *
    *    gefuehrt.                                                       *
    *                                                                    *
    *  e n t r i e s :                                                   *
    *    PROCEDURE do_add;                                               *
    *    PROCEDURE do_sub;                                               *
    *    PROCEDURE do_transp;                                            *
    *    PROCEDURE do_mult;                                              *
    *    PROCEDURE do_det;                                               *
    *    PROCEDURE do_inv;                                               *
    *    PROCEDURE do_lu;                                                *
    *    PROCEDURE do_lin;                                               *
    *    PROCEDURE do_qr;                                                *
    *    PROCEDURE do_rank;                                              *
    *    PROCEDURE do_cholesky;                                          *
    *    PROCEDURE do_eye;                                               *
    *    PROCEDURE do_rand;                                              *
    *                                                                    *
    **********************************************************************)


PROCEDURE do_add;

   VAR

      i, j    :integer;

BEGIN
   load_Intern (Eingabe.Nr[2], M1);
   load_Intern (Eingabe.Nr[3], M2);
   IF (M1.n <> M2.n) or (M1.m <> M2.m) THEN BEGIN
      N_Error (62);
   END
   ELSE BEGIN
      M3.Name:= Result_Name;
      M3.n:= M1.n;
      M3.m:= M1.m;
      FOR i:= 1 TO M1.n DO BEGIN
         FOR j:= 1 TO M1.m DO BEGIN
            M3.x[i,j]:= M1.x[i,j] + M2.x[i,j];
         END;
      END;
      save_Intern (M3);
   END;
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_sub;

   VAR
      i, j    :integer;

BEGIN
   load_Intern (Eingabe.Nr[2], M1);
   load_Intern (Eingabe.Nr[3], M2);
   IF (M1.n <> M2.n) or (M1.m <> M2.m) THEN BEGIN
      N_Error (62);
   END
   ELSE BEGIN
      M3.Name:= Result_Name;
      M3.n:= M1.n;
      M3.m:= M1.m;
      FOR i:= 1 TO M1.n DO BEGIN
         FOR j:= 1 TO M1.m DO BEGIN
            M3.x[i,j]:= M1.x[i,j] - M2.x[i,j];
         END;
      END;
      save_Intern (M3);
   END;
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_transp;

   VAR
      i, j    :integer;

BEGIN
   load_Intern (Eingabe.Nr[2], M1);
   M2.Name:= Result_Name;
   M2.n:= M1.m;
   M2.m:= M1.n;
   FOR i:= 1 TO M1.n DO BEGIN
      FOR j:= 1 TO M1.m DO BEGIN
         M2.x[j,i]:= M1.x[i,j];
      END;
   END;
   save_Intern (M2);
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_mult;
BEGIN
   load_Intern (Eingabe.Nr[2], M1);
   load_Intern (Eingabe.Nr[3], M2);
   IF (M1.m <> M2.n) THEN BEGIN
      N_Error (62);
      exit;
   END;
   M3.Name:= Result_Name;
   M3.n:= M1.n;
   M3.m:= M2.m;
   Matrix_Mult (M1.x, M2.x, M3.x, M1.n, M1.m, M2.n, M2.m);
   save_Intern (M3);
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_det;

      VAR
         singulaer    :boolean;
         index, f     :integer;
         Determinante :real;

BEGIN
   Load_Intern (Eingabe.Nr[2], M1);
   IF M1.m <> M1.n THEN BEGIN
      N_Error (60);
      exit;
   END;
   LU_Decomp (M1.x, M2.x, M1.n, f, singulaer);
   IF singulaer THEN BEGIN
      N_Error (63);
      exit;
   END;
   Determinante:= 1.0;
   FOR Index:= 1 TO M1.n DO BEGIN
      IF (ln(abs(Determinante)) + ln(abs(Determinante)))/ln(10) >= 38 THEN BEGIN
         N_Error (21);
         exit;
      END
      ELSE BEGIN
         Determinante:= Determinante * M1.x[index,index];
      END;
   END;
   M2.m:= 1;
   M2.n:= 1;
   M2.Name:= Result_Name;
   M2.x[1,1]:= Determinante * f;
   save_Intern (M2);
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_inv;

   VAR
      singulaer  :boolean;

BEGIN
   Load_Intern (Eingabe.Nr[2], M1);
   IF (M1.n <> M1.m) THEN BEGIN
      N_Error (60);
      exit;
   END;
   Matrix_Inverse (M1.x, M2.x, M1.n, singulaer);
   (*  M1 enthaelt nun die invertierte Matrix  *)
   (*  M2 enthaelt nun die Pivotmatrix  *)
   IF singulaer THEN BEGIN
      N_Error (63);
      exit;
   END;
   M1.Name:= Result_Name;
   M2.Name:= Name_Pivot_Matrix;
   M2.n:= M1.n;
   M2.m:= M1.m;
   Save_Intern (M1);
   Save_Intern (M2);
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_lu;

   VAR
       singulaer  :boolean;
       i, j, f    :integer; (* f wird nur fuer den Procedureaufruf verwendet *)
       det        :real;
       ipvt       :T_integer_Vektor;

BEGIN
   Load_Intern (Eingabe.Nr[2], M1);
   IF (M1.n <> M1.m) THEN BEGIN
      N_Error (62);
      exit;
   END;
   LU_decomp (M1.x, M3.x, M1.n, f, singulaer);
   (*  M1 wird zerlegte Matrix  *)
   (*  M3 wird Pivot - Matrix  *)
   IF singulaer THEN BEGIN
      N_Error (63);
      exit;
   END;
   FOR i:= 1 TO M1.n DO BEGIN
      FOR j:= 1 TO M1.n DO BEGIN
         IF i>j THEN BEGIN
            M2.x[i,j]:= M1.x[i,j];
            M1.x[i,j]:= 0;
         END;
         IF i<j THEN BEGIN
            M2.x[i,j]:= 0;
         END;
         IF i=j THEN BEGIN
            M2.x[i,j]:= 1;
         END;
      END;
   END;
   M1.Name:= Name_U_Matrix;
   M2.Name:= Name_L_Matrix;
   M3.Name:= Name_Pivot_Matrix;
   M2.n:= M1.n;
   M2.m:= M1.m;
   M3.n:= M1.n;
   M3.m:= M1.m;
   save_Intern (M2);
   save_Intern (M1);
   save_Intern (M3);
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_lin;

   VAR
       singulaer   :boolean;
       f           :integer;       (* f nur fuer Procedureaufruf *)
       ipvt        :T_integer_Vektor;

BEGIN
   Load_Intern (Eingabe.Nr[2], M1);
   Load_Intern (Eingabe.Nr[3], M2);
   IF (M1.n <> M1.m) or (M2.n <> M1.n) or (M2.m <> 1) THEN BEGIN
      N_Error (62);
      exit;
   END;
   LU_decomp (M1.x, M3.x, M1.n, f, singulaer);
   (*  M3 wird die Pivotmatrix      *)
   (*  M1 wird die zerlegte Matrix  *)
   IF singulaer THEN BEGIN
      N_Error (63);
      exit;
   END;
   Message_Enable:= false;
   Matrix_Mult (M3.x, M2.x, M2.x, M1.n, M1.m, M2.n, M2.m);
   Message_Enable:= true;
   (*  M2 wird die Pivotierte Resultatmatrix  *)
   Solve (M1.x, M2.x, M1.n);
   (*  M2 enthaelt die Loesungen  *)
   M2.Name:= Result_Name;
   M3.Name:= Name_Pivot_Matrix;
   M3.n:= M1.n;
   M3.m:= M1.m;
   save_Intern (M2);
   save_Intern (M3);
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE Do_QR;

   var
      singulaer    :boolean;

BEGIN
   Load_Intern (Eingabe.Nr[2], M1);
   IF M1.n <> M1.m THEN BEGIN
      N_Error (60);
      exit;
   END;
   QR_Decomp (M1.x, M2.x, M1.n, M1.m, singulaer);
   IF singulaer THEN BEGIN
      N_Error (63);
      exit;
   END;
   M1.Name:= Name_R_Matrix;
   Save_Intern (M1);
   M2.Name:= Name_Q_Matrix;
   M2.n:= M1.n;
   M2.m:= M1.n;
   Save_Intern (M2);
   N_Message (9);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_rank;
BEGIN
   N_Message (90);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_cholesky;

   VAR
      mistake    :boolean;
      posdef     :boolean;
      n, i, j, k :integer;

BEGIN
   load_Intern (Eingabe.Nr[2], M1);
   IF (M1.n <> M1.m) THEN BEGIN
      N_Error (60);
      exit;
   END;
   n:= M1.n;
   mistake:= false;
   FOR i:= 1 TO n DO BEGIN
      FOR j:= i+1 TO n DO BEGIN
         IF (M1.x[i,j] <> M1.x[j,i]) and (not mistake) THEN BEGIN
            N_Error (66);
            mistake:= true;
         END;
         M1.x[j,i]:= 0;
      END;
   END;
   posdef:= true;
   i:= 1;
   WHILE (i < n) and (M1.x[i,i] > epsilon) DO BEGIN
      M1.x[i,i]:= sqrt (M1.x[i,i]);
      FOR k:= i+1 TO n DO BEGIN
         M1.x[i,k]:= M1.x[i,k]/M1.x[i,i];
      END;
      FOR j:= i+1 TO n DO BEGIN
         FOR k:= j TO n DO BEGIN
            M1.x[j,k]:= M1.x[j,k]-M1.x[i,j]*M1.x[i,k];
         END;
      END;
      i:= i+1;
   END;
   IF (i = k) and (M1.x[n,n] > epsilon) THEN BEGIN
      M1.x[n,n]:= sqrt(M1.x[n,n]);
      M1.Name:= Result_Name;
      save_Intern (M1);
      N_Message (9);
   END
   ELSE BEGIN
      N_Error (65);
   END;
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_eye;
BEGIN
   M1.Name:= Eingabe.Arg[2];
   Window_off;
   M1.n:= N_integer_Ask (15);
   M1.m:= M1.n;
   Window_on;
   Matrix_Eye (M1.x, M1.n);
   save_Intern (M1);
END;

(*------------------------------------------------------------------------*)

PROCEDURE do_rand;

   VAR
      i, j     :integer;

BEGIN
   M1.Name:= Eingabe.Arg[2];
   Window_off;
   M1.n:= get_m_integer ('lines :  ', 1, max_Ind);
   M1.m:= get_m_integer ('columns :  ', 1, max_Ind);
   Window_on;
   FOR i:= 1 TO M1.n DO BEGIN
      FOR j:= 1 TO M1.m DO BEGIN
         M1.x[i,j]:= 2 * Random - 1;
      END;
   END;
   save_Intern (M1);
   N_Message (9);
END;

